﻿-- Macro Profiles
local MPS_Version = "1.5"

MPS = LibStub("AceAddon-3.0"):NewAddon("Profiles: Macros", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")

local MAX_ACCOUNT_MACROS, MAX_CHARACTER_MACROS = 36, 18

--Char specific macros
minMacros = MAX_ACCOUNT_MACROS + 1
maxMacros = MAX_ACCOUNT_MACROS+MAX_CHARACTER_MACROS

function MPS:OnInitialize()
   MPdb = LibStub("AceDB-3.0"):New("ProfilesMacros",nil,"Default")
end

function MPS:Echo(text)
   DEFAULT_CHAT_FRAME:AddMessage("Profiles: Macros - "..text)
end

-- ProfilesHub: Send message to all slaves that we are now saving profile
function MPS:PreSaveMacroProfile(name)
	MPS:SendAceMsg("save",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function MPS:SaveMacroProfile(name)
	-- No char macros!
	local _,numperchar = GetNumMacros()
	if numperchar == 0 then MPS:Echo("Nothing to save!") return end
	
	MPdb:SetProfile(name)
	
	-- Solving /mp save tank X 2
	if MPS:ExistsProfile(name) then MPdb:ResetProfile(name) end
	
	MPS:Echo("Saving macros to profile "..name.."!")
	
	local icons_table = {}

	local numIcons = GetNumMacroIcons()
	for i=1,numIcons do
		local lowercase_icons = string.lower(GetMacroIconInfo(i)) -- Normalize icon path from the API
		icons_table[lowercase_icons] = i
	end
    
	for i=minMacros,maxMacros do
		local macro_name, texture, body = GetMacroInfo(i)
		local actionBar_pos = {};
		
		-- No more macros saved
		if not macro_name then break end
			
		-- Get icon ID instead of path (needed for CreateMacro() in Load!
		texture = string.lower(texture)
        if icons_table[texture] then 
             texture = icons_table[texture]
        end
		
		-- Icon not found, give question mark!
		if (type(texture) ~= "number") then 
			texture = '1'  
		end

		-- Save actionbar position
		for actionID=1, 120 do
			local _, id = GetActionInfo(actionID)
			local abmacro_name, _, _ = GetMacroInfo(id)
			
			if abmacro_name == macro_name then
				table.insert(actionBar_pos , actionID)
			end
		end
		
		-- Save the macro to db
		MPdb.profile[i] = { macro_name,texture,body,actionBar_pos }
	end
		
	MPS:SendMessage("MPS_Profiles_save",name) -- ProfilesHub
end

-- This is an alias for ProfilesHub
function MPS:Save(...)
	MPS:SaveMacroProfile(...)
end

function MPS:LoadMacroProfile(name)

	-- No such profile saved
	if not MPS:ExistsProfile(name) then MPS:Echo(name.." profile not found.") return end
	
	-- In combat!
	if InCombatLockdown() then MPS:Echo("Cannot change macros in combat!") return end

   MPdb:SetProfile(name)
  
   MPS:Echo("Loading macros from profile "..name.."!")
   
   --Get rid of all the current macros
   for i=minMacros,maxMacros do
      DeleteMacro(minMacros) -- DeleteMacro changes index for all other macros
   end 
   
   --Load saved macros profile
   for index, keys in pairs(MPdb.profile) do   
       if keys[1] then    
            if (type(keys[2]) ~= "number") then keys[2] = '1' end -- Icon not found, give question mark!
            
            CreateMacro(keys[1], keys[2], keys[3], 1);
			-- TODO: If Profiles: Action bars then let_PAB_handle_it end
			-- Action bar handling
			for i,v in pairs(keys[4]) do
				ClearCursor()
				
				-- Clear current spot
				PickupAction(v)
				ClearCursor()
				
				-- Restore this spot
				PickupMacro(keys[1])
				PlaceAction(v)
			end
			
       end
   end
   
	if (IsAddOnLoaded("Blizzard_MacroUI")) then
		MacroFrame_Update() -- Blizzards own code
	end
   
	MPS:SendMessage("MPS_Profiles_load",name) -- ProfilesHub
end

-- This is an alias for ProfilesHub
function MPS:Load(...)
	MPS:LoadMacroProfile(...)
end

function MPS:ExistsProfile(name)
   for i, k in ipairs(MPdb:GetProfiles()) do
      local key = k
      
      if key == name then
         return (key)
      end
   end
   return nil
end


SLASH_MP1 = "/mp"
SlashCmdList["MP"] = function(msg)
   local cmd, arg = string.split(" ", msg, 2)
   cmd = string.lower(cmd or "")
   arg = arg or ""
   
   if (cmd == "version") then
      print("Profiles: Macros v."..MPS_Version)
   elseif (cmd=="list")then
      for i,k in ipairs(MPdb:GetProfiles()) do
		if k == MPdb:GetCurrentProfile() then
			print(' - ' .. k.." (current)")
		else
			print(' - ' .. k)
		end
	end
   elseif (cmd=="" or arg=="")then
      print("/mp delete <name> * Not implemepted yet")
      print("/mp save <name>")
      print("/mp load <name>")
      print("/mp list")
      print("/mp version")
      print("/mp debug [on|off] * Not implemepted yet")
   elseif (cmd=="save" and arg~="")then
      MPS:PreSaveMacroProfile(arg)
   elseif (cmd=="load" and arg~="")then
      MPS:LoadMacroProfile(arg)
   elseif (cmd=="delete" and arg~="")then
	print("Profiles - Macros: Not implemented yet!")
   else    
      print("wrong entry")
   end
end

--Regexp routine
-- nocase("Hi there!")) >>> 
-- [hH][iI] [tT][hH][eE][rR][eE]!
function MPS:nocase (s)
   s = string.gsub(s, "%a", function (c)
         return string.format("[%s%s]", string.lower(c),
            string.upper(c))
   end)
   return s
end